import React from 'react'

import {
  Alert,
  AlertTitle,
  Badge,
  Box,
  Button,
  Divider,
  HStack,
  Icon,
  IconButton,
  Select,
  Spinner,
  Tooltip,
  useToast,
} from '@chakra-ui/react'
import { FaArrowLeft } from 'react-icons/fa'

import { Link, back, navigate, routes, useLocation } from '@redwoodjs/router'
import { LANGUAGES } from 'src/constants'
import { useTranslation } from 'react-i18next'
import { getInitialLanguageCode } from 'src/i18n'
import { ablyService } from 'src/services/ably-service'
import { useTestsRunningStore } from 'src/pages/implementations/run-tests/tests-running-store'
import { Report } from 'src/models'
import { match, P } from 'ts-pattern'
import { capitalize } from 'lodash'

const REPORTS_CHANNEL_NAME = 'report'

type ReportMessage = {
  reportId?: Report['id']
  status: 'SUCCESS' | 'ERROR'
}
export const TOPBAR_HEIGHT_PX = '40px'
export const Topbar: React.FC = () => {
  const store = useTestsRunningStore()
  const toast = useToast()
  const { isRunning } = store
  const { t, i18n } = useTranslation()
  const { pathname } = useLocation()
  const initialLanguageCode = getInitialLanguageCode()

  const renderSuccessReportToast = (reportId: Report['id']) => {
    toast({
      status: 'success',
      title: 'hola',
      duration: 9000,
      render: () => (
        <Alert status="info">
          <AlertTitle>{capitalize(t('new_report_available'))}</AlertTitle>
          {/* @ts-ignore -- TODO */}
          <Button onClick={() => navigate(routes.report({ id: reportId }))}>
            {capitalize(t('view'))}
          </Button>
        </Alert>
      ),
    })
  }

  React.useEffect(() => {
    const channel = ablyService.getChannel(REPORTS_CHANNEL_NAME)

    const listener = (message) => {
      const reportMessage = JSON.parse(message.data as string) as ReportMessage
      match(reportMessage)
        .with(
          { status: 'SUCCESS', reportId: P.not(P.nullish) },
          ({ reportId }) => {
            store.setIsRunning(false)
            renderSuccessReportToast(reportId)
          }
        )
        .otherwise(() => {
          toast({
            status: 'error',
            duration: 9000,
            title: capitalize(t('new_report_error')),
          })
          store.setIsRunning(false)
        })
    }

    channel.subscribe('event', listener)

    return () => channel.unsubscribe('event', listener)
  }, [])

  const handleOnBack = () => {
    if (pathname === '/') {
      // @ts-ignore -- TODO
      navigate(routes.home())
    } else {
      back()
    }
  }

  const onChangeLang = (e: React.ChangeEvent<HTMLSelectElement>) => {
    const langCode = e.target.value
    i18n.changeLanguage(langCode)
    localStorage.setItem('language', langCode)
  }

  return (
    <>
      <HStack
        h={TOPBAR_HEIGHT_PX}
        w="full"
        justifyContent="space-between"
        pr="4"
        bgColor="blue.200"
        position="sticky"
        top="0"
        spacing="0"
      >
        {pathname !== '/' ? (
          <IconButton
            aria-label="back"
            variant="unstyled"
            onClick={handleOnBack}
            icon={<Icon as={FaArrowLeft} color="gray.600" />}
          />
        ) : (
          <Box />
        )}

        <HStack>
          {isRunning && (
            <Box pt="1">
              <Tooltip label="Se está generando el reporte">
                <Spinner size="sm" />
              </Tooltip>
            </Box>
          )}
          {/* @ts-ignore -- TODO */}
          <Link to={routes.patterns()}>
            <Badge textAlign="right" color="gray.600" variant="-2u">
              {t('patterns_patterns')}
            </Badge>
          </Link>
          <Select
            onChange={onChangeLang}
            defaultValue={initialLanguageCode}
            variant="unstyled"
            fontSize="xs"
          >
            {LANGUAGES.map(({ code, label }) => (
              <option key={code} value={code}>
                {label}
              </option>
            ))}
          </Select>
        </HStack>
      </HStack>
      <Divider />
    </>
  )
}
