import {
  AlertDialog,
  ButtonProps,
  Tooltip,
  useDisclosure,
  useToast,
  Text,
  AlertDialogOverlay,
  AlertDialogContent,
  AlertDialogHeader,
  AlertDialogBody,
  AlertDialogFooter,
  Button,
} from '@chakra-ui/react'
import React from 'react'
import { useQueryClient } from 'react-query'
import { Animation, Pattern } from 'src/models'
import { DeleteAnimationMutation, GetAnimationsQuery } from '../query-builder'
import { useTranslation } from 'react-i18next'
import { capitalize } from 'lodash'

export type DeleteAnimationButtonProps = {
  animation: Animation
  patternId: Pattern['id']
  hasTooltip?: boolean
  children: React.ReactElement<ButtonProps>
}

export const DeleteAnimationButton: React.FC<DeleteAnimationButtonProps> = ({
  animation,
  patternId,
  hasTooltip = false,
  children,
}) => {
  const { t } = useTranslation()
  const toast = useToast()
  const queryClient = useQueryClient()
  const [isLoading, setIsLoading] = React.useState(false)
  const { isOpen, onOpen, onClose } = useDisclosure()
  const cancelRef = React.useRef<HTMLButtonElement>(null)

  const deleteAnimationMutation = DeleteAnimationMutation.useMutation({
    onSuccess: () => {
      setIsLoading(false)
      toast({
        status: 'success',
        position: 'top',
        title: capitalize(t('adnimations_delete_success')),
      })
      queryClient.invalidateQueries(GetAnimationsQuery.getKey({ patternId }))
    },
    onError: () => {
      toast({
        status: 'error',
        position: 'top',
        title: `Ha ocurrido un error al eliminar la animación.`,
      })
      onClose()
      setIsLoading(false)
    },
  })

  const handleOnDelete = () => {
    setIsLoading(true)
    deleteAnimationMutation.mutate({ id: animation.id })
  }

  if (!React.isValidElement(children) || Array.isArray(children)) {
    throw new Error(
      'DeleteAnimationButton child must be a single clickable element'
    )
  }

  const btn = React.cloneElement(children, { onClick: onOpen })

  return (
    <>
      <Tooltip
        hasArrow
        shouldWrapChildren
        isDisabled={!hasTooltip}
        label={<Text>{capitalize(t('animations_delete'))}</Text>}
      >
        {btn}
      </Tooltip>
      <AlertDialog
        isOpen={isOpen}
        onClose={onClose}
        leastDestructiveRef={cancelRef}
      >
        <AlertDialogOverlay>
          <AlertDialogContent>
            <AlertDialogHeader fontSize="lg" fontWeight="bold">
              {capitalize(t('animations_delete'))}
            </AlertDialogHeader>
            <AlertDialogBody>
              <Text>
                {capitalize(
                  t('animations_delete_animation_confirmation_modal_body')
                )}
              </Text>
            </AlertDialogBody>
            <AlertDialogFooter>
              <Button ref={cancelRef} variant="ghost" onClick={onClose}>
                {capitalize(t('cancel'))}
              </Button>
              <Button
                variant="solid"
                onClick={handleOnDelete}
                ml="3"
                isLoading={isLoading}
              >
                {capitalize(t('delete'))}
              </Button>
            </AlertDialogFooter>
          </AlertDialogContent>
        </AlertDialogOverlay>
      </AlertDialog>
    </>
  )
}
