import {
  ButtonProps,
  Modal,
  ModalContent,
  ModalHeader,
  ModalOverlay,
  useDisclosure,
  Text,
  ModalCloseButton,
  ModalBody,
  VStack,
  Tooltip,
  FormControl,
  FormLabel,
  Input,
  FormErrorMessage,
  Textarea,
  ModalFooter,
  HStack,
  Button,
  Icon,
  ModalProps,
} from '@chakra-ui/react'
import React from 'react'
import { Pattern } from 'src/models'
import { CreateImplementationMutation } from '../query-builder'
import { useFormik } from 'formik'
import {
  CreateImplementationSchema,
  createImplementationSchema,
} from './schema'
import { useQueryClient } from 'react-query'
import { GetImplementationsByPatternIdQuery } from '../query-builder/get-implementations-by-pattern-id-query'
import { FaInfo } from 'react-icons/fa'
import { useTranslation } from 'react-i18next'
import { capitalize } from 'lodash'

const FORM_ID = 'add-implementation-form'

export type AddImplementationModalProps = {
  pattern: Pattern
  disclosure: ReturnType<typeof useDisclosure>
} & Partial<ModalProps>

export const AddImplementationModal: React.FC<AddImplementationModalProps> = ({
  pattern,
  disclosure,
  ...props
}) => {
  const { t } = useTranslation()
  const queryClient = useQueryClient()
  const createImplementationMutation = CreateImplementationMutation.useMutation(
    {
      onSuccess: () => {
        queryClient.invalidateQueries(
          GetImplementationsByPatternIdQuery.getKey({ patternId: pattern.id })
        )
      },
    }
  )

  const handleOnSubmit = async (values: CreateImplementationSchema) => {
    await createImplementationMutation.mutateAsync({
      name: values.name,
      description: values.description,
      patternId: pattern.id,
      dockerhubImageName: values.dockerhubImageName,
    })

    disclosure.onClose()
  }

  const {
    handleSubmit,
    handleReset,
    handleChange,
    setTouched,
    values,
    isSubmitting,
    errors,
    touched,
    isValid,
  } = useFormik({
    enableReinitialize: true,
    initialValues: {
      name: '',
      description: '',
      dockerhubImageName: '',
    },
    validateOnBlur: true,
    validateOnChange: true,
    validateOnMount: true,
    validationSchema: createImplementationSchema,
    onSubmit: handleOnSubmit,
  })

  return (
    <>
      {disclosure.isOpen && (
        <Modal {...disclosure} {...props}>
          <ModalOverlay />
          <ModalContent>
            <ModalHeader>
              <Text fontSize="md">
                {capitalize(
                  t('implementations_add_implementation_for_pattern')
                )}{' '}
                {pattern.name}
              </Text>
            </ModalHeader>
            <ModalCloseButton />
            <form id={FORM_ID} onSubmit={handleSubmit} onReset={handleReset}>
              <ModalBody>
                <VStack>
                  <FormControl isInvalid={Boolean(errors.name) && touched.name}>
                    <FormLabel>
                      <HStack>
                        <Text fontSize="sm">{capitalize(t('name'))}</Text>
                        <Text color="red">*</Text>
                      </HStack>
                    </FormLabel>
                    <Input
                      name="name"
                      autoFocus
                      value={values.name}
                      onChange={(e) => {
                        setTouched({ ...touched, name: true })
                        handleChange(e)
                      }}
                    />
                    <FormErrorMessage>{errors.name}</FormErrorMessage>
                  </FormControl>

                  <FormControl
                    isInvalid={
                      Boolean(errors.dockerhubImageName) &&
                      touched.dockerhubImageName
                    }
                  >
                    <FormLabel>
                      <HStack>
                        <Text fontSize="sm">
                          {capitalize(
                            t('implementations_dockerhub_image_name')
                          )}
                        </Text>
                        <Text color="red">*</Text>
                        <Tooltip
                          label={capitalize(
                            t('implementations_dockerhub_image_name_tooltip')
                          )}
                          shouldWrapChildren
                          hasArrow
                        >
                          <Icon as={FaInfo} color="blue.500" />
                        </Tooltip>
                      </HStack>
                    </FormLabel>
                    <Input
                      name="dockerhubImageName"
                      autoFocus
                      value={values.dockerhubImageName}
                      onChange={(e) => {
                        setTouched({ ...touched, dockerhubImageName: true })
                        handleChange(e)
                      }}
                    />
                    <FormErrorMessage>
                      {errors.dockerhubImageName}
                    </FormErrorMessage>
                  </FormControl>

                  <FormControl
                    isInvalid={
                      Boolean(errors.description) && touched.description
                    }
                  >
                    <FormLabel>
                      <HStack>
                        <Text fontSize="sm">
                          {capitalize(t('description'))}
                        </Text>
                        <Text color="red">*</Text>
                      </HStack>
                    </FormLabel>
                    <Textarea
                      name="description"
                      value={values.description}
                      onChange={(e) => {
                        setTouched({ ...touched, description: true })
                        handleChange(e)
                      }}
                    />
                    <FormErrorMessage>{errors.description}</FormErrorMessage>
                  </FormControl>
                </VStack>
              </ModalBody>
            </form>

            <ModalFooter>
              <HStack>
                <Button variant="outline" onClick={disclosure.onClose}>
                  {capitalize(t('cancel'))}
                </Button>
                <Button
                  form={FORM_ID}
                  variant="solid"
                  colorScheme="blue"
                  type="submit"
                  isLoading={isSubmitting}
                  isDisabled={!isValid}
                >
                  {capitalize(t('save'))}
                </Button>
              </HStack>
            </ModalFooter>
          </ModalContent>
        </Modal>
      )}
    </>
  )
}
